class Entry {
  // data attributes
  String title = "";
  int year = 0; 
  int month = 0;
  int agent_index = 0;
  
  // positional attributes
  int[] counts = new int[TIME_POINTS];
  PVector[] positions = new PVector[TIME_POINTS];
  PVector currpos = new PVector(0, 0, 0);
  
  // viz attributes
  color c = color(0, 0, 0); 
  float size = 20.0;
  float avgCount = 0.0;
  
  Entry(int agent_index, String title, int year, int month, int ct) {
    this.title = title;
    this.year = year;
    this.month = month;
    this.agent_index = agent_index;
    
    for (int i = 0; i < TIME_POINTS; i++) {
      counts[i] = 0; // set counts to 0 for all time points
      positions[i] = new PVector(0.0, 0.0, 0.0); // set positions to 0 for all time points
    }
  }
  
  void storeCounts(int i, int ct) { 
    //BY THE TIME THIS IS CALLED, NUM_AGENTS and TIME_POINTS should be all computed
    if ( i >= 0 && i < TIME_POINTS) {
      counts[i] = ct;
    }
  }
  
  void mapPosition() { // after counts are all stored, call this to map counts to proper position of the agents on screen
    float avg = 0.0;
    for (int i = 0; i < TIME_POINTS; i++) {
      avg += counts[i];
      positions[i].x = map(agent_index, 0, NUM_AGENTS, 0, MAX_X); // which unique entry it is, mapped on screen to x pos
      positions[i].y = map(counts[i], 0, MAX_COUNT, 0, MAX_Y); // which count value does the entry have, mapped on screen to y pos
      positions[i].z = map(i, 0, TIME_POINTS, 0, MAX_Z); // which time point are we on, mapped on screen to z pos
    }  
    currpos.x = positions[0].x; 
    currpos.y = positions[0].y;
    currpos.z = positions[0].z;
    avg /= TIME_POINTS; // this is the average count per agent
    avgCount = avg; // store in the class
    
    c = color( 0, 0, 255 ); //map(avgCount, 0, MAX_COUNT, 200, 255)
    size = map(counts[time_index], 0, MAX_COUNT, 1, 50);
  }
 
  boolean updatePosition(int time_index) { // agent movement
    size = map(counts[time_index], 0, MAX_COUNT, 1, 50);
    currpos.z = positions[time_index].z;
    
    PVector distance = new PVector(positions[time_index].x - currpos.x, 
                                   positions[time_index].y - currpos.y, 
                                   positions[time_index].z - currpos.z);
    
    if (abs(distance.y) > system_rate) { // if it is below the target
      if (distance.y > 0) { currpos.y += system_rate; }
      else if (distance.y < 0) { currpos.y -= system_rate; }
      else {currpos = positions[time_index]; return true;}
      return false;
    } else { currpos = positions[time_index]; return true;}
  }

  void drawAgent() { // agent visualization
    pushMatrix();
    if (agent_index == unique_entry) { fill(255, 255, 255); }
    else { fill(c); }
    noStroke();
    translate(currpos.x, currpos.y, currpos.z);
    rotateX(cam.getRotations()[0]); //rotate towards the camera angle in X direction
    rotateY(cam.getRotations()[1]); //rotate towards the camera angle in X direction
    rotateZ(cam.getRotations()[2]); //rotate towards the camera angle in X direction
    ellipse(0, 0, size,size);
    //sphere(size);
    popMatrix();
  }
 
  void drawAgentHistory(int time_index) { // agent history visualization
    for (int i = 0; i < time_index - 1; i++) {
      pushMatrix();
      noStroke();
      if (agent_index == unique_entry) { fill(color(255, 255, map(i, 0, time_index, 20, 100), 255));
      } else { fill(color(0, 0, map(i, 0, time_index, 50, 200), 255), map(avgCount, 0, MAX_COUNT, 100, 255)); }
      translate(positions[i].x, positions[i].y, positions[i].z);
      rotateX(cam.getRotations()[0]); //rotate towards the camera angle in X direction
      rotateY(cam.getRotations()[1]); //rotate towards the camera angle in X direction
      rotateZ(cam.getRotations()[2]); //rotate towards the camera angle in X direction
      //sphere(map(counts[i], 0, MAX_COUNT, 1, 50));
      ellipse(0, 0, map(counts[i], 0, MAX_COUNT, 1, 50), map(counts[i], 0, MAX_COUNT, 1, 50));
      popMatrix();
      
      
      //stroke(0, 0, map(i, 0, time_index, 50, 255), 255);
      if (agent_index == unique_entry) { stroke(255, 255, map(i, 0, time_index, 50, 255), 255);
      } else { stroke(0, 0, map(i, 0, time_index, 50, 255), 255); }
      strokeWeight( map(counts[i], 0, MAX_COUNT, 1, 3) );
      if (i >= 1) {
        line(positions[i-1].x, positions[i-1].y, positions[i-1].z,
             positions[i].x, positions[i].y, positions[i].z);
      } else { 
        line(positions[i].x, 0, positions[i].z, 
             positions[i].x, positions[i].y, positions[i].z); 
      }
    }
    
    if (time_index > 0) {
      line(positions[time_index-1].x, positions[time_index-1].y, positions[time_index-1].z, 
             currpos.x, currpos.y, currpos.z);
    }
  }
  
  void drawLabels() {
    // display title of agent
    //if (mouseX >= currpos.x - size && mouseX <= currpos.x + size) { // hover
      pushMatrix();
      translate(currpos.x, 0, 0);
      rotateZ(radians(90));
      if (agent_index == unique_entry) { fill(255, 255, 255); }
      else { fill(0, 0, 255); }
      textAlign(LEFT);
      textSize(30);
      scale(0.2);
      text(title, 50, 0, 0);
      popMatrix();
    //}
  }
}
